library WriteLine;

uses
  SysUtils,
  Windows,
  Classes,
  Controls,
  Dlg_WriteFile in 'Dlg_WriteFile.pas' {FileDLG};

{$R *.RES}

Const  Inputs = 4;  // number of inputs
       Outputs = 0; // number of outputs made adjustable

       {INPUTS}
       CLK   = 0;   // index of input variable CLK
       RST   = 1;   // index of input variable RST
       FName = 2;
       SX    = 3;

       {OUTPUTS}
       SOUT = 0;

       {USER}
       CLK_old  = 0;          // index of user variable CLK_old
       RST_old  = 1;          // index of user variable RST_old
       SELECT   = 2;
       MERKER   = 3;
       DLLINDEX = 100;

Type TDLLParams = array[0..100] of extended;   //Type of ProfiLab DLL parameters
     PDLLParams = ^TDLLParams;                 // Pointer to ProfiLab DLL parameters

Type TStringParams = array[0..100] of PChar;
     PStringParams = ^TStringParams;


function NumInputs: Byte;
begin
  result:= Inputs; //Define FIXED number of component input pins
end;

function NumOutputs: Byte; // Made adjustable with 'NumOutputsEx'
begin
  result:= Outputs; //Define FIXED number of component output pins
end;

Function InputName(Channel: Byte): ShortString; // Return name for each component input pin
begin
   case Channel of
   CLK:   result:='/WR';   //  "WRCLK" (Clock)
   RST:   result:='/RST';  //  "/RST" (NOT RESET)
   FName: result:= 'File';
   SX:    result:= '$I';
   end;
end;

Function OutputName(Channel: Byte): ShortString; // Return name for each component output pin
begin
   result:='$O';
end;

Procedure SimStart(PInput,POutput,PUser: PDLLParams; PStrings: PStringParams); //called when ProfiLab enters RUN mode
var Filename,Datenstring : string;
    Datei : TEXT;
begin
  PUser^[RST_old]:= 0;
  PInput^[CLK]:= 0;
  PUser^[MERKER]:= 0;
end;

Procedure SimStop(PInput,POutput,PUser: PDLLParams); //called when ProfiLab RUN mode is terminated
begin
  // nothing to be done
end;

Procedure CalculateEx(PInput,POutput,PUser: PDLLParams; PStrings: PStringParams); //called regularly from ProfiLab
var Filename,Datenstring : string;
    Datei : TEXT;
begin
  Filename:= AnsiString(PStrings[FName]);
  if filename <> '' then
  begin
    Datenstring:= AnsiString(PStrings[SX]);
//    StrPCopy(PStrings[SOUT],PChar(Filename));

    assignfile(Datei,Filename);

    if (round(PUser^[MERKER]) = 0) then  //nur einmal ausfuehren
    begin
      PUser^[MERKER]:= 5;
      if (round(PUser^[SELECT]) = 0) then //wenn ueberschreiben gewaehlt wurde
      begin //reset
        rewrite(Datei);
        if ioresult = 0 then CloseFile(Datei);
      end;
    end;

    if PInput^[RST] < 2.5 then //check RST input HIGH or LOW
    begin
      if (not (PInput^[RST] >= 2.5)) and (PUser^[RST_old] > 2.5) then //check out falling edge at RST input
      begin //Reset
        rewrite(Datei);
        if ioresult = 0 then CloseFile(Datei);
      end;
      PUser^[RST_old]:= PInput^[RST]; //Remember RST status for next call
      exit;
    end;
    PUser^[RST_old]:= PInput^[RST]; //Remember RST status for next call

    if PInput^[CLK] < 2.5 then //check CLK input HIGH or LOW
    begin
      if (not (PInput^[CLK] >= 2.5)) and (PUser^[CLK_old] > 2.5) then //check out rising edge at CLK input
      begin //write
        append(Datei);
        if ioresult <> 0 then rewrite(Datei);
        if ioresult = 0 then
        begin
          writeln(Datei,Datenstring);
          CloseFile(Datei);
        end;
      end;
    end;
    PUser^[CLK_old]:= PInput^[CLK]; //Remember CLK status for next call
  end;
end;

Procedure Configure(PUser: PDLLParams);
var DLG: TFileDLG;
begin
  DLG:= TFileDLG.Create(nil);                       //Create user dialogue
  With DLG do
  begin
    Caption:='WriteLine [DLL'+FloatToStr(PUser^[DLLIndex])+']'; // Show ProfiLab DLL-Index in dialogue caption
    BitsCombo.ItemIndex:=round(PUser^[SELECT]);       // Set dialogue values

    if ShowModal = mrOK then                                 // Execute dialogue
    begin
      PUser^[SELECT]:= BitsCombo.ItemIndex;           // Read back values from dialogue;
    end;

    free;                                                  // Dont forget to free dialogue
  end;
end;


//export methods for ProfiLab
exports SimStart,
        SimStop,
        NumInputs,
        NumOutputs,
        CalculateEx,
        InputName,
        OutputName,
        Configure;
begin
end.

